-- A set of enemies that spawn as a group
-- TODO: initialize waves from a file so we can define levels

import "CoreLibs/object"
import "entity"

local gfx <const> = playdate.graphics
local geom <const> = playdate.geometry

class("Wave").extends()

function Wave.new()
   local w = Wave()
   return w
end

function Wave:init()
   Wave.super.init(self)
   self.entities = {}
end

-- Adds an entity to the wave. Positions are playdate.geometry.point objects
-- - position is where the entity should end up at the start of the combat sequence
-- - vec is the enemy's starting vector
-- - prePosition is the entity's start location for "flying in" animation
-- - introDuration is the number of milliseconds the animation should take. Use this to control the fly-in speed
function Wave:addEntity(entity)
   table.insert(self.entities, entity)
end

-- Add all the sprites in the Wave to the global table
-- Also initializes their intro animations
function Wave:add()
   for i=1, #self.entities, 1 do
      self.entities[i]:add()
   end
end

-- Call this every frame for an active Wave
-- It will crawl the Entity list and remove any "dead"
-- entities, and will return true to signal the wave end
-- when all enemies are defeated
function Wave:update()
   local dead = {}
   for i=1, #self.entities, 1 do
      if self.entities[i].health == 0 then
         table.insert(dead, i)
         print(string.format("%d is dead", i))
      end
   end

   -- Starting with the highest index to avoid renumbering errors,
   -- we remove dead entities from the list
   while #dead > 0 do
      table.remove(self.entities, table.remove(dead))
   end

   if #self.entities == 0 then
      return true
   end

   return false
end

